#ifndef LIBPV_H
#define LIBPV_H

#include <stdio.h>
#include <inttypes.h>

#define PV_PATH_MAX (255)

typedef struct {
	char	_data[52];
} pv_engine_t;


typedef enum pv_bs_store_type {
	PV_STORE_STANDARD,
	PV_STORE_8BIT,
} pv_store_type_t;


typedef struct pv_engine_attr {
	char			xml_filename[PV_PATH_MAX + 1];
	char			store_filename[PV_PATH_MAX + 1];
	pv_store_type_t		store_type;
} pv_engine_attr_t;


/* start/stop/stuff */
extern int	pv_init			(pv_engine_t		*engine,
					 pv_engine_attr_t	*attr);

extern int	pv_destroy		(pv_engine_t		*engine_anon);

extern int	pv_engine_attr_init	(pv_engine_attr_t	*attr);

extern int	pv_engine_attr_init_from_cmdline
					(pv_engine_attr_t	*attr,
					 int			argc,
					 char			*argv[]);

extern int	pv_debuglevel		(pv_engine_t		*engine,
					 int			level);


/* registration stuff */
extern uint8_t  *pv_register_uint8	(pv_engine_t		*engine,
					 const char		*varname);

extern uint16_t *pv_register_uint16	(pv_engine_t		*engine,
					 const char		*varname);

extern uint32_t *pv_register_uint32	(pv_engine_t		*engine,
					 const char		*varname);

extern char     *pv_register_string	(pv_engine_t		*engine,
					 const char		*varname);


/* string handling stuff */
extern int	pv_cmp_string		(pv_engine_t		*engine,
					 const char		*string,
					 char			*s2);

extern int	pv_get_string		(pv_engine_t		*engine,
					 const char		*string,
					 char			*dest,
					 size_t			n);

extern int	pv_set_string		(pv_engine_t		*engine,
					 char			*string,
					 char			*src,
					 size_t			n);

/* set-a-default */
extern int	pv_default		(pv_engine_t		*engine,
					 void			*var);

/* lock-stuff */
extern void	pv_lock			(pv_engine_t		*engine);
extern void	pv_unlock		(pv_engine_t		*engine);
extern void	pv_flush		(pv_engine_t		*engine,
					 void			*var,
					 size_t			n);

#endif	/* !LIBPV_H */
