#include <libpv_config.h>

#include <errno.h>
#include <string.h>

#include <libpv.h>
#include <pv_types.h>
#include <pv_bs.h>

int pv_cmp_string(pv_engine_t	*engine_anon,
		  const char	*pv_string,
		  char		*s2)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	struct pv		*pv;
	int			finished = 0;

	if (!engine || !engine->pv_list)
		return INT32_MIN;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
		
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (engine->store.map + pv->offset == pv_string) {
			return strncmp(pv_string, s2, pv->size);
		}
	}

	return INT32_MIN;
}


int pv_get_string(pv_engine_t	*engine_anon,
		  const char	*pv_string,
		  char		*dest,
		  size_t	n)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	struct pv		*pv;
	int			finished = 0;
	size_t			size;

	if (!engine || !engine->pv_list)
		return -EINVAL;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
		
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (engine->store.map + pv->offset == pv_string) {
			size = MIN(pv->size, n);
			strncpy(dest, pv_string, size);
			dest[size] = 0;
			return size;
		}
	}

	return 0;
}


/*
 * this functions flushes!
 */
int pv_set_string(pv_engine_t	*engine_anon,
		  char		*pv_string,
		  char		*src,
		  size_t	n)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	struct pv		*pv;
	int			finished = 0;
	size_t			size;

	if (!engine || !engine->pv_list)
		return -EINVAL;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
		
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (engine->store.map + pv->offset == pv_string) {
			size = MIN(pv->size, n);
			strncpy(pv_string, src, size);
			pv_string[size] = 0;

			pv_bs_flush(engine, pv_string, pv->size);
			return size;
		}
	}

	return 0;
}
